AnimationClip = Core.class(Sprite)

function AnimationClip:init(texturepack, animations, step)
	
	local clipFrames	= {}
	local texturePack	= TexturePack.new(texturepack..".txt", texturepack..".png", true)
	local frameIterator	= 0
	
	self.frameIndexes	= {}
	self.movieclip		= nil
	self.handler		= nil
	
	for action, frames in pairs(animations) do
		
		for _, frame in pairs(frames) do
			
			local frameSprite = Bitmap.new(texturePack:getTextureRegion(frame))
			frameSprite:setAnchorPoint(0.5, 0.5)
			
			local frameStart	= frameIterator + 1
			local frameEnd		= frameIterator + step
			
			table.insert(clipFrames, {frameStart, frameEnd, frameSprite})
			
			frameIterator = frameEnd
			
		end
		
		self.frameIndexes[action] = {frameIterator - ((#frames * step) - 1), frameIterator}
		
	end
	
	self.movieclip = MovieClip.new(clipFrames)
	
	self:addChild(self.movieclip)

end

function AnimationClip:onComplete(handler)
	
	if self.handler then return end
	
	self.handler = handler
	
	self.movieclip:addEventListener(Event.COMPLETE, self.handler)
	
end

function AnimationClip:onComplereRemove()

	self.movieclip:removeEventListener(Event.COMPLETE, self.handler)
	
end

function AnimationClip:play(action, pos, loop)
	
	local pos	= pos or false
	local loop	= loop or false
	
	if pos and pos > 0 then
		
		self.movieclip:setScale(1, 1, 1)
		
	end
	
	if pos and pos < 0 then
		
		self.movieclip:setScale(-1, 1, 1)
		
	end
	
	if loop then
		
		self.movieclip:setGotoAction(self.frameIndexes[action][2], self.frameIndexes[action][1])
		
	else
		
		self.movieclip:setStopAction(self.frameIndexes[action][2])
		
	end
	
	self.movieclip:gotoAndPlay(self.frameIndexes[action][1])
	
end