Enemy = Core.class(UnitBase)

function Enemy:init()
	
	self.props = self:assignProperties({}, {
		hp		= 1,
		pos		= 0,
		range	= 10000,
		type	= 3
	})
	
	self.width			= 48
	self.height			= 48
	self.jumps			= {now = 0, max = 1}
	self.hp				= {now = self.props.hp, prev = self.props.hp}
	self.speed			= {def = 5, now = 0, last = self.props.pos}
	self.sensor			= {foot = 0, left = 0, right = 0}
	
	self.clip		= AnimationClip.new(
		"EnemyTexture",
		{
			walk	= {"ENEMY_WALK_R_1.png","ENEMY_WALK_R_2.png","ENEMY_WALK_R_1.png","ENEMY_WALK_R_3.png"},
			jump	= {"ENEMY_JUMP_R_1.png","ENEMY_JUMP_R_2.png","ENEMY_JUMP_R_3.png","ENEMY_JUMP_R_2.png"},
			idle	= {"ENEMY_STAY_R_1.png","ENEMY_STAY_R_1.png","ENEMY_STAY_R_1.png","ENEMY_STAY_R_1.png","ENEMY_STAY_R_1.png","ENEMY_STAY_R_1.png","ENEMY_STAY_R_1.png","ENEMY_STAY_R_1.png","ENEMY_STAY_R_1.png","ENEMY_STAY_R_1.png","ENEMY_STAY_R_1.png","ENEMY_STAY_R_1.png","ENEMY_STAY_R_1.png","ENEMY_STAY_R_1.png","ENEMY_STAY_R_1.png","ENEMY_STAY_R_2.png","ENEMY_STAY_R_3.png","ENEMY_STAY_R_2.png"},
			attack	= {"ENEMY_SHOOT_R_1.png", "ENEMY_SHOOT_R_1.png"}
		}, 10
	)
	
	self:addChild(self.clip)
	
	local body						= world.world:createBody({type = b2.DYNAMIC_BODY})
	
	local halfW = self.width / 2
	local halfH	= self.height / 2
	local slope	= 2
	--[[
	local bodyShape				= B2Shape:polygon({polygon = {
		{x = -halfW, y = -halfH + slope},
		{x = -halfW + slope, y = -halfH},
		{x = halfW - slope, y = -halfH},
		{x = halfW, y = -halfH + slope},
		{x = halfW, y = halfH - slope},
		{x = halfW - slope, y = halfH},
		{x = -halfW + slope, y = halfH},
		{x = -halfW, y = halfH - slope}
	}})
	]]
	
	local bodyShape					= B2Shape:rectangle({width = self.width, height = self.height})
	self.fixture					= body:createFixture({shape = bodyShape, density = 5, friction = 0, restitution = 0})
	
	local footShape					= B2Shape:rectangle({width = self.width - 2, height = 4}, 0, self.height / 2 + 2)
	self.sensorFootFixture			= body:createFixture({shape = footShape, density = 0, friction = 0, restitution = 0, isSensor = true})
	self.sensorFootFixture.ft		= true

	local sensorRightShape			= B2Shape:rectangle({width = 3, height = 15}, 26, 30)
	self.sensorRightFixture			= body:createFixture({shape = sensorRightShape, density = 0, friction = 0, restitution = 0, isSensor = true})
	self.sensorRightFixture.sr		= true
	
	local sensorLeftShape			= B2Shape:rectangle({width = 3, height = 15}, -26, 30)
	self.sensorLeftFixture			= body:createFixture({shape = sensorLeftShape, density = 0, friction = 0, restitution = 0, isSensor = true})
	self.sensorLeftFixture.sl		= true
	
	body:setPosition(self.x, self.y)
	self:setPosition(self.x, self.y)

	body:setFixedRotation(true)
	
	self.body	= body
	body.object	= self
	--[
	local fixtureFilter				= {categoryBits = 1, maskBits = 1, groupIndex = -1}
	
	self.fixture:setFilterData(fixtureFilter)
	self.sensorFootFixture:setFilterData(fixtureFilter)
	self.sensorRightFixture:setFilterData(fixtureFilter)
	self.sensorLeftFixture:setFilterData(fixtureFilter)
	--]]
	self:addEventListener(Event.ENTER_FRAME, self.move, self)
	--self:addEventListener(Event.ENTER_FRAME, self.chasePlayer, self)
	self:walkRight()
end

function Enemy:walkLeft()
	
	if self.speed.now < 0 then return end
	
	self.speed.now	= -self.speed.def
	self.speed.last	= self.speed.now

	self:animateWalk()
	
end

function Enemy:walkRight()

	if self.speed.now > 0 then return end
	
	self.speed.now	= self.speed.def
	self.speed.last	= self.speed.now

	self:animateWalk()

end


function Enemy:move()
	
	local vx, vy = self.body:getLinearVelocity()
	self.body:setLinearVelocity(self.speed.now, vy)

end


