UnitBase = Core.class(ObjectBase)

function UnitBase:init()
	
	
end

function UnitBase:isOnScreen()
	
	local ofsX, ofsY	= self.interact.camera:getOffset()
	local zoom			= self.interact.camera:getZoom()
	local pX, pY		= self:getX(), self:getY()
	local hW			= self:getWidth() / 2
	local hH			= self:getHeight() / 2
	
	if
		pX + hW < -ofsX or pX - hW > -ofsX + appSettings.w / zoom or
		pY + hH < -ofsY or pY - hH > -ofsY + appSettings.h / zoom
	then
		return false
	end
	
	return true
	
end

function UnitBase:isShadowed()
	
	local isShadowed	= false
	local allLights		= self.world:getObjectGroup("light")
	
	for k, light in pairs(allLights) do
		
		local sX, sY			= self:getPosition()
		local tX, tY			= light:getPosition()
		local range				= (sX - tX) ^ 2 + (sY - tY) ^ 2
		
		self.isShadowedClosest	= 1
		
		self.isShadowedCastFunction	= self.isShadowedCastFunction or function(fixture, hitX, hitY, vectX, vectY, fraction)
			
			if
				not fixture:isSensor() and
				fraction < self.isShadowedClosest and
				fixture:getBody().object.ext and
				fixture:getBody().object.ext.shade and
				tonumber(fixture:getBody().object.ext.shade) == 1
			then
				
				self.isShadowedClosest = fraction
				
			end
			
			return 1
			
		end
		
		self.world.world:rayCast(sX, sY, tX, tY, self.isShadowedCastFunction)
		
		if self.isShadowedClosest < 1 and range <= (light.power * 0.9) ^ 2 then
			
			isShadowed = true
			
		end
		
	end
	
	return isShadowed
	
end

function UnitBase:castVision(target, segments, offset, color)
	
	local dbg		= false
	local area		= nil
	local color	 	= color or 0xff1000
	local offset 	= offset
	local tX, tY 	= target[1], target[2]
	local sX, sY 	= self:getX() + offset[1], self:getY() + offset[2]
	local dR	 	= (sX - tX) ^ 2 + (sY - tY) ^ 2
	local ang	 	= ^>(math.atan2(tY - sY, tX - sX)) + 180
	local segments 	= segments or {{0, 360, 100}}
	local detector	= false
	
	self.visibleObstacles = self.visibleObstacles or TablePool.new()
	
	self.visibleObstacles:free()
	
	if dbg then
		
		area = Shape.new()
		area:setFillStyle(Shape.SOLID, color)
		area:beginPath()
		area:lineTo(sX, sY)
		for key, val in ipairs(segments) do
			for k = val[1], val[2] do
				area:lineTo(math.cos(^<(k - 180)) * val[3] + sX, math.sin(^<(k - 180)) * val[3] + sY)
			end
			area:lineTo(sX, sY)
		end
		area:lineTo(sX, sY)
		area:closePath()
		area:endPath()
		area:setAlpha(0.3)
		Timer.delayedCall(1, function() area:removeFromParent() end)
		self.world:addChild(area)
		
	end
	
	for s = 1, #segments do
		local val = segments[s]
		if val[1] <= ang and ang <= val[2] and dR <= val[3] ^ 2 and not detector then detector = true end
	end
	
	if
		not detector or
		(tX == sX and tY == sY)
	then
		return false
	end
	
	if dbg then
		
		local trace	= Shape.new()
		trace:setLineStyle(2, 0x00FF00)
		trace:beginPath()
		trace:lineTo(sX, sY)
		trace:lineTo(tX, tY)
		trace:closePath()
		trace:endPath()
		Timer.delayedCall(1, function() trace:removeFromParent() end)
		self.world:addChild(trace)
		
	end
	
	self.castVisionClosest = self.castVisionClosest or {}
	
	self.castVisionClosest.f = 1
	self.castVisionClosest.x = tX
	self.castVisionClosest.y = tY
	
	self.castVisionCastFunction = self.castVisionCastFunction or function(fixture, hitX, hitY, vectX, vectY, fraction)
		
		local obj = fixture:getBody().object
		
		if
			not fixture:isSensor() and
			fraction <= self.castVisionClosest.f and
			obj ~= self
		then
			
			if obj.ext and obj.ext.transparent then
				
				local obstacle = self.visibleObstacles:get()
				
				obstacle = obj
				
				return -1
				
			end
			
			local oX, oY = obj.body:getPosition()
			
			self.castVisionClosest.f = fraction
			self.castVisionClosest.x = oX
			self.castVisionClosest.y = oY
			
		end
		
	end
	
	self.world.world:rayCast(sX, sY, tX, tY, self.castVisionCastFunction)
	
	if self.castVisionClosest.x == tX and self.castVisionClosest.y == tY then
		
		if dbg then
			
			area:setAlpha(0.7)
			
		end
		
		if next(self.visibleObstacles:pool()) then
			
			return self.visibleObstacles:pool()
			
		else
			
			return true
			
		end
		
	end
	
	return false

end

function UnitBase:setColor(hp)
	
	local r, g, b	= 1, 1, 1
	
	if hp <= 5 then
		r, g, b	= 1, 1, 4
	end
	
	if hp <= 3 then
		r, g, b	= 1, 4, 1
	end

	if hp <= 1 then
		r, g, b	= 4, 1, 1
	end
	
	self.clip:setColorTransform(r, g, b)
	
end

function UnitBase:animateHit()
	
	self.blinkAnimation = self.blinkAnimation or GTween.new(self.clip, 0.1, {redMultiplier = 2}, {repeatCount = 4, reflect = true, ease = easing.inOutQuadratic})
	
	self.tweens:addTween(
		self.blinkAnimation, "blink"
	)
	
	self.hitPhysicsDef.x[1] = self:getX()
	self.hitPhysicsDef.y[1] = self:getY()
	
	PhysicsParticleEmitter.new(self.world, BloodParticle.new, 10, self.hitPhysicsDef, self.hitParticleDef)
	
end

function UnitBase:animateDeath()
	
	self.hitPhysicsDef.x[1] = self:getX()
	self.hitPhysicsDef.y[1] = self:getY()
	
	PhysicsParticleEmitter.new(self.world, BloodParticle.new, 10, self.hitPhysicsDef, self.hitParticleDef)
	PhysicsParticleEmitter.new(self.world, BoneParticle.new, 10, self.hitPhysicsDef, self.hitParticleDef)
	
	if self.lastContact and self.lastContact.object.type == "sword" then
		
		local wOfsX, wOfsy	= self.interact.camera:getOffset()
		local wZoom			= self.interact.camera:getZoom()
		
		self.splashDef.xTo	= -wOfsX + appSettings.w / 2 / wZoom
		self.splashDef.yTo	= -wOfsy + appSettings.h / 2 / wZoom
		self.splashDef.x	= self:getX()
		self.splashDef.y	= self:getY()
		
		self.world:addChild(
			Splash.new(self.splashList, self.splashDef)
		)
		
	end

end

function UnitBase:animateShoot()
	
	if self.speed.last > 0 then
		
		self.clip:play("attack", 1, false)
		
	else
		
		self.clip:play("attack", -1, false)
		
	end
	
	self.clip:onComplete(function()
		self:animateWalk()
	end)
	
end

function UnitBase:animateWalk()
	
	-- Is on floor
	if(self.sensor.foot > 0) then
		
		-- Is walking right
		if(self.speed.now > 0) then
			
			self.clip:play("walk", 1, true)
			
		-- Is walking left
		elseif(self.speed.now < 0) then
			
			self.clip:play("walk", -1, true)
			
		-- Is not walking
		else
			
			-- Last speed dir
			if(self.speed.last > 0) then
				
				self.clip:play("idle", 1, true)
				
			else
				
				self.clip:play("idle", -1, true)
				
			end
			
		end
		
	else
		
		-- Last speed dir
		if(self.speed.last > 0) then
			
			self.clip:play("jump", 1, true)
			
		else
			
			self.clip:play("jump", -1, true)
			
		end
		
	end
	
end
--[[
function UnitBase:animateWalk()
	
	-- Is on floor
	if(self.sensor.foot > 0) then
		
		-- Is walking right
		if(self.speed.now > 0) then
			
			self.clip:setGotoAction(400, 361)
			self.clip:gotoAndPlay(361)
			
		-- Is walking left
		elseif(self.speed.now < 0) then
			
			self.clip:setGotoAction(440, 401)
			self.clip:gotoAndPlay(401)
			
		-- Is not walking
		else
			
			-- Last speed dir
			if(self.speed.last > 0) then
				
				self.clip:setGotoAction(180, 1)
				self.clip:gotoAndPlay(1)
				
			else
				
				self.clip:setGotoAction(360, 181)
				self.clip:gotoAndPlay(181)
				
			end
			
		end
		
	else
		-- Last speed dir
		if(self.speed.last > 0) then
			
			self.clip:setGotoAction(480, 441)
			self.clip:gotoAndPlay(441)
			
		else
			
			self.clip:setGotoAction(520, 481)
			self.clip:gotoAndPlay(481)
			
		end
		
	end
	
end

function UnitBase:animateShoot()
	
	if self.sensor.foot > 0 then
		
		if self.speed.last > 0 then
			
			self.clip:setStopAction(530)
			self.clip:gotoAndPlay(521)
			
		else
			
			self.clip:setStopAction(540)
			self.clip:gotoAndPlay(531)
			
		end
		
	else
		
		if self.speed.last > 0 then
			
			self.clip:setStopAction(550)
			self.clip:gotoAndPlay(541)
			
		else
			
			self.clip:setStopAction(560)
			self.clip:gotoAndPlay(551)
			
		end
		
	end
	
	self.clip:addEventListener(Event.COMPLETE, function()
		self:animateWalk()
	end)
	
end
--]]
function UnitBase:getState()
	
	local pX, pY = self.body:getPosition()
	
	return {
		hp = {now = self.hp.now},
		px = pX,
		py = pY,
		pl = self.speed.last,
	}
	
end

function UnitBase:setState(stateData)
	
	if stateData.hp then
		self.hp.now		= stateData.hp.now
		self.hp.prev	= stateData.hp.now
	end
	
	if stateData.pl then
		self.speed.last = stateData.pl
	end
	
	if stateData.px and stateData.py and self.type == "player" then
		self.body:setPosition(stateData.px, stateData.py)
		self:setPosition(stateData.px, stateData.py)
	end

end